// Copyright by Uri Siman, IAPT - 1999, Delphi 2 main module
unit Pro;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  QuickRep;

const Siz = 50; //max count fields in row
      SizY = 40; //max (band) height of page in Pixels/ab1

type
   TOnPrintEvent = procedure(Sender : TObject; var Value : String) of object;
   TAfterPrintEvent = procedure(BandPrinted : Boolean) of object;
   TRepBeforePrintEvent = procedure(var ReportPrint : Boolean) of object;
   TRepAfterPrintEvent = procedure(Sender: TObject) of object;

SendPageWordOn = Class
 OldCall : Array[0..Siz*5] of TOnPrintEvent;
 PrnBand : Array[0..Siz*5] of TAfterPrintEvent;
 end;

type
  TWordTable = class (TPersistent)
   private
    FTableAutoFormat : Word;
    FTableAutoFit : Boolean;
    FTableRealWidth : Boolean;
    FDetailToTable : Boolean;
    FDetailMultyRows : Boolean;
    FTitleToTable : Boolean;
    FPageHeaderToTable : Boolean;
    FPageFooterToTable : Boolean;
    FGroupHeaderToTable : Boolean;
    FGroupFooterToTable : Boolean;
    FSubDetailToTable : Boolean;
    FColumnHeaderToTable : Boolean;
    FOverlayToTable : Boolean;
    FSummaryToTable : Boolean;
   published
    property TableAutoFormat : Word read FTableAutoFormat write FTableAutoFormat default 16;
    property TableAutoFit : Boolean read FTableAutoFit write FTableAutoFit;
    property TableRealWidth : Boolean read FTableRealWidth write FTableRealWidth;
    property DetailToTable : Boolean read FDetailToTable write FDetailToTable;
    property DetailMultyRows : Boolean read FDetailMultyRows write FDetailMultyRows;
    property TitleToTable : Boolean read FTitleToTable write FTitleToTable;
    property PageHeaderToTable : Boolean read FPageHeaderToTable write FPageHeaderToTable;
    property PageFooterToTable : Boolean read FPageFooterToTable write FPageFooterToTable;
    property GroupHeaderToTable : Boolean read FGroupHeaderToTable write FGroupHeaderToTable;
    property GroupFooterToTable : Boolean read FGroupFooterToTable write FGroupFooterToTable;
    property SubDetailToTable : Boolean read FSubDetailToTable write FSubDetailToTable;
    property ColumnHeaderToTable : Boolean read FColumnHeaderToTable write FColumnHeaderToTable;
    property OverlayToTable : Boolean read FOverlayToTable write FOverlayToTable;
    property SummaryToTable : Boolean read FSummaryToTable write FSummaryToTable;
  end;

type
  TWordParams = class (TPersistent)
   private
    FIgnorePageHeader : Boolean;
    FIgnorePageFooter : Boolean;
    FTopMargin : Word;
    FBottomMargin : Word;
    FLeftMargin : Word;
    FRightMargin : Word;
    FLineSpacing : String;
    FFileName : String;
    FAutoClose : Boolean;
    FScaleFontSize : Integer;
    FHeaderDistance : String;
    FFooterDistance : String;
    FFontNameTabs : TFontName;
    FFontSizeTabs : Word;
   published
    property IgnorePageHeader : Boolean read FIgnorePageHeader write FIgnorePageHeader;
    property IgnorePageFooter : Boolean read FIgnorePageFooter write FIgnorePageFooter;
    property TopMargin : Word read FTopMargin write FTopMargin default 20;
    property BottomMargin : Word read FBottomMargin write FBottomMargin default 20;
    property LeftMargin : Word read FLeftMargin write FLeftMargin default 20;
    property RightMargin : Word read FRightMargin write FRightMargin default 20;
    property LineSpacing : String read FLineSpacing write FLineSpacing;
    property FileName : String read FFileName write FFileName;
    property AutoClose : Boolean read FAutoClose write FAutoClose;
    property ScaleFontSize : Integer read FScaleFontSize write FScaleFontSize default 0;
    property HeaderDistance : String read FHeaderDistance write FHeaderDistance;
    property FooterDistance : String read FFooterDistance write FFooterDistance;
    property FontNameTabs : TFontName read FFontNameTabs write FFontNameTabs;
    property FontSizeTabs : Word read FFontSizeTabs write FFontSizeTabs default 14;
  end;

type
  TTransferScale = class (TPersistent)
   private
    FTabA : Integer;
    FTabB : Integer;
    FMainA : Integer;
    FMainB : Integer;
   published
    property TabA : Integer read FTabA write FTabA default 32;
    property TabB : Integer read FTabB write FTabB default 33;
    property MainA : Integer read FMainA write FMainA default 1;
    property MainB : Integer read FMainB write FMainB default 1;
  end;

type
  TStencil = class (TPersistent)
   private
    FHorizontalSensitiv : Word;
    FVerticalSensitiv : Word;
   published
    property VerticalSensitiv : Word read FVerticalSensitiv write FVerticalSensitiv default 14;
    property HorizontalSensitiv : Word read FHorizontalSensitiv write FHorizontalSensitiv default 3;
  end;

type
  TTabBorders = class (TPersistent)
   private
    FBorderInside : Boolean;
    FBorderOutside : Boolean;
   published
    property BorderInside : Boolean read FBorderInside write FBorderInside;
    property BorderOutside : Boolean read FBorderOutside write FBorderOutside;
  end;


  TDRep = class(TComponent)
  private
    { Private declarations }
    FExecute : Boolean;
    FShowExecute : Boolean;
    FSeparator : Char;
    FWordTable : TWordTable;
    FWordParams : TWordParams;
    FTransferScale : TTransferScale;
    FStencil : TStencil;
    FTabBorders : TTabBorders;
  protected
    { Protected declarations }
  public
    { Public declarations }
    PageWordOn : SendPageWordOn;
    procedure QRLabel1Print(sender: TObject; var Value: string);
    procedure QRBand1AfterPrint(BandPrinted: Boolean);
    procedure APrint(sender: TObject);
    procedure BPrint(var PrintReport : Boolean);
    procedure Loaded; override;
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Execute : Boolean read FExecute write FExecute;
    property ShowExecute : Boolean read FShowExecute write FShowExecute;
    property Separator : Char read FSeparator write FSeparator default #9;
    property WordTable : TWordTable read FWordTable write FWordTable;
    property WordParams : TWordParams read FWordParams write FWordParams;
    property TransferScale : TTransferScale read FTransferScale write FTransferScale;
    property Stencil : TStencil read FStencil write FStencil;
    property TabBorders : TTabBorders read FTabBorders write FTabBorders;
  end;

var ExecuteToWord : Boolean;
    Bevent        : Boolean;
    tqr : TQuickReport;
     fm : TForm;
    Ab1 : Word; {14 : Vertical Sensitivity in Pixels}
    Ab2 : Word; {4 : Horizontal Sensitivity in Pixels, 3-optimal}
 OldRef : Word;
// RefCnt : Word;

procedure Register;

Function PagePack(Y, L : Integer) : Word;

implementation
uses OleAuto, Printers, ProSub;

Var
   BefPrn : TRepBeforePrintEvent;
   AftPrn : TRepAfterPrintEvent;

constructor TDRep.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 FStencil:=TStencil.Create;
 FWordTable:=TWordTable.Create;
 FWordParams:=TWordParams.Create;
 FTransferScale:=TTransferScale.Create;
 FTabBorders:=TTabBorders.Create;

 FSeparator:=#9;
 FShowExecute:= False;

 With FStencil Do
 Begin
  FVerticalSensitiv:= 14;
  FHorizontalSensitiv:= 4;
 end;

 With FWordParams Do
 Begin
  FIgnorePageHeader:= True;
  FIgnorePageFooter:= True;
  FTopMargin:= 22;
  FBottomMargin:= 22;
  FLeftMargin:= 22;
  FRightMargin:= 22;
  FLineSpacing:= '1.3';
  FFileName:= 'RepWord';
  FAutoClose:= False;
  FScaleFontSize:= 0;
  FHeaderDistance:='1.5';
  FFooterDistance:='1.5';
  FFontNameTabs:='Arial';
  FFontSizeTabs:= 8;
 end;

 With FTransferScale Do
 Begin
  TabA:= 32;
  TabB:= 33;
  MainA:= 1;
  MainB:= 1;
 end;

 With FWordTable Do
 Begin
  FTableAutoFormat:=16;
  FDetailToTable:= False;
  FDetailMultyRows:= False;
  FTableAutoFit:= True;
  FTableRealWidth:=True;
  FTitleToTable:= False;
  FPageHeaderToTable:= False;
  FPageFooterToTable:= False;
  FGroupHeaderToTable:= False;
  FGroupFooterToTable:= False;
  FSubDetailToTable:= False;
  FColumnHeaderToTable:= False;
  FOverlayToTable:= False;
  FSummaryToTable:= False;
 end;

 With FTabBorders Do
 Begin
  FBorderInside:= True;
  FBorderOutSide:= True;
 end;

// RefCnt :=0; //nothing row of tables before
 OldRef :=0; //nothing band before
 PrnPage :=0; //printed pages

end;

destructor TDRep.Destroy;
begin
 ExecuteToWord:=FALSE;
 QRPrinter.CleanUp;

 FStencil.Free;
 FWordTable.Free;
 FWordParams.Free;
 FTransferScale.Free;
 FTabBorders.Free;

 inherited Destroy;
end;

Function PagePack(Y, L : Integer) : Word;
Var I, J : Integer;
Begin
 For I:=0 to PageWord.Empty[Y]-1 Do
 If PageWord.Left[I, Y] >= L then
 Begin
   For J:=PageWord.Empty[Y] Downto I+1 Do
   Begin
    PageWord.WLine[J, Y]:= PageWord.WLine[J-1, Y];
    PageWord.Top[J, Y]:= PageWord.Top[J-1, Y];
    PageWord.Left[J, Y]:= PageWord.Left[J-1, Y];
    PageWord.Width[J, Y]:= PageWord.Width[J-1, Y];
    PageWord.FName[J, Y]:= PageWord.FName[J-1, Y];
    PageWord.FSize[J, Y]:= PageWord.FSize[J-1, Y];
    PageWord.FStyle[J, Y]:= PageWord.FStyle[J-1, Y];
   end;

   If PageWord.Empty[Y]<Siz then
      PageWord.Empty[Y]:=PageWord.Empty[Y]+1;
   Result:=I;
   Exit;
 end;

 If PageWord.Empty[Y]<Siz then
    PageWord.Empty[Y]:=PageWord.Empty[Y]+1;

 Result:=PageWord.Empty[Y] - 1;
end;

procedure TDRep.QRLabel1Print(sender: TObject; var Value: string);
var str : string;
   X, Y : Integer;
    Pix : Integer;
     Tg : LongInt;
begin

If Length(Value)<=0 then
If ((TQRBand((Sender As Tcontrol).Parent).BandType=rbTitle)
  And (FWordTable.FTitleToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbDetail)
  And (FWordTable.FDetailToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbPageHeader)
  And (FWordTable.FPageHeaderToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbPageFooter)
  And (FWordTable.FPageFooterToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbGroupHeader)
  And (FWordTable.FGroupHeaderToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbGroupFooter)
  And (FWordTable.FGroupFooterToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbSubDetail)
  And (FWordTable.FSubDetailToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbColumnHeader)
  And (FWordTable.FColumnHeaderToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbOverlay)
  And (FWordTable.FOverlayToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbSummary)
  And (FWordTable.FSummaryToTable=True)) then
    Value:=' ';

 Case PageWord.Typ[(Sender As TComponent).Tag] of
 1: If Assigned(PageWordOn.OldCall[(Sender As TQRLabel).Tag]) then
    Begin
     {  ,  }
     (Sender As TQRLabel).OnPrint:=
                 PageWordOn.OldCall[(Sender As TQRLabel).Tag];
      Tg:=(Sender As TQRLabel).Tag; {  }
     (Sender As TQRLabel).Tag:= PageWord.OldTag[Tg];
     {  }
     (Sender As TQRLabel).OnPrint(sender, Value);
     (Sender As TQRLabel).Tag:=Tg;
    end;
 2: If Assigned(PageWordOn.OldCall[(Sender As TQRDBText).Tag]) then
    Begin
     {  ,  }
     (Sender As TQRDBText).OnPrint:=
                 PageWordOn.OldCall[(Sender As TQRDBText).Tag];
      Tg:=(Sender As TQRDBText).Tag; {  }
     (Sender As TQRDBText).Tag:= PageWord.OldTag[Tg];
     {  }
     (Sender As TQRDBText).OnPrint(sender, Value);
     (Sender As TQRDBText).Tag:=Tg;
    end;
 3: If Assigned(PageWordOn.OldCall[(Sender As TQRDBCalc).Tag]) then
    Begin
     {  ,  }
     (Sender As TQRDBCalc).OnPrint:=
                 PageWordOn.OldCall[(Sender As TQRDBCalc).Tag];
      Tg:=(Sender As TQRDBCalc).Tag; {  }
     (Sender As TQRDBCalc).Tag:= PageWord.OldTag[Tg];
     {  }
     (Sender As TQRDBCalc).OnPrint(sender, Value);
     (Sender As TQRDBCalc).Tag:=Tg;
    end;
 4: If Assigned(PageWordOn.OldCall[(Sender As TQRSysData).Tag]) then
    Begin
     {  ,  }
     (Sender As TQRSysData).OnPrint:=
                 PageWordOn.OldCall[(Sender As TQRSysData).Tag];
      Tg:=(Sender As TQRSysData).Tag; {  }
     (Sender As TQRSysData).Tag:= PageWord.OldTag[Tg];
     {  }
     (Sender As TQRSysData).OnPrint(sender, Value);
     (Sender As TQRSysData).Tag:=Tg;
    end;
  end;

  Ref:=PageWord.Band[(Sender As TComponent).Tag];

  If (Sender Is TQRLabel) then
  Begin
   {   }
   Y:= (Sender As TQRLabel).Top div Ab1;
   X:= (Sender As TQRLabel).Left div Ab2;
   PageWord.Name[(Sender As TComponent).Tag]:=
                 (Sender As TQRLabel).Name;

   X:=PagePack(Y, (Sender As TQRLabel).Left);

   PageWord.WLine[X, Y]:= Value;
   PageWord.Top[X, Y]:= (Sender As TQRLabel).Top;
   PageWord.Left[X, Y]:= (Sender As TQRLabel).Left;
   PageWord.Width[X, Y]:= (Sender As TQRLabel).Width;
   PageWord.FName[X, Y]:= (Sender As TQRLabel).Font.Name;
   PageWord.FSize[X, Y]:= (Sender As TQRLabel).Font.Size;
   PageWord.FStyle[X, Y]:= (Sender As TQRLabel).Font.Style;
   end;

  If (Sender Is TQRDBText) then
  Begin
   {   }
   Y:= (Sender As TQRDBText).Top div Ab1;
   X:= (Sender As TQRDBText).Left div Ab2;
   PageWord.Name[(Sender As TComponent).Tag]:=
                 (Sender As TQRDBText).Name;

   X:=PagePack(Y, (Sender As TQRDBText).Left);

   PageWord.WLine[X, Y]:= Value;
   PageWord.Top[X, Y]:= (Sender As TQRDBText).Top;
   PageWord.Left[X, Y]:= (Sender As TQRDBText).Left;
   PageWord.Width[X, Y]:= (Sender As TQRDBText).Width;
   PageWord.FName[X, Y]:= (Sender As TQRDBText).Font.Name;
   PageWord.FSize[X, Y]:= (Sender As TQRDBText).Font.Size;
   PageWord.FStyle[X, Y]:= (Sender As TQRDBText).Font.Style;
  end;

  If (Sender Is TQRDBCalc) then
  Begin
   {   }
   Y:= (Sender As TQRDBCalc).Top div Ab1;
   X:= (Sender As TQRDBCalc).Left div Ab2;
   PageWord.Name[(Sender As TComponent).Tag]:=
                 (Sender As TQRDBCalc).Name;

   X:=PagePack(Y, (Sender As TQRDBCalc).Left);

   PageWord.WLine[X, Y]:= Value;
   PageWord.Top[X, Y]:= (Sender As TQRDBCalc).Top;
   PageWord.Left[X, Y]:= (Sender As TQRDBCalc).Left;
   PageWord.Width[X, Y]:= (Sender As TQRDBCalc).Width;
   PageWord.FName[X, Y]:= (Sender As TQRDBCalc).Font.Name;
   PageWord.FSize[X, Y]:= (Sender As TQRDBCalc).Font.Size;
   PageWord.FStyle[X, Y]:= (Sender As TQRDBCalc).Font.Style;
   end;

  If (Sender Is TQRSysData) then
  Begin
   {   }
   Y:= (Sender As TQRSysData).Top div Ab1;
   X:= (Sender As TQRSysData).Left div Ab2;
   PageWord.Name[(Sender As TComponent).Tag]:=
                 (Sender As TQRSysData).Name;

   X:=PagePack(Y, (Sender As TQRSysData).Left);

   PageWord.WLine[X, Y]:= Value;
   PageWord.Top[X, Y]:= (Sender As TQRSysData).Top;
   PageWord.Left[X, Y]:= (Sender As TQRSysData).Left;
   PageWord.Width[X, Y]:= (Sender As TQRSysData).Width;
   PageWord.FName[X, Y]:= (Sender As TQRSysData).Font.Name;
   PageWord.FSize[X, Y]:= (Sender As TQRSysData).Font.Size;
   PageWord.FStyle[X, Y]:= (Sender As TQRSysData).Font.Style;
   end;
  try
   RefW:=TQRBand((Sender As Tcontrol).Parent).Height;
  except
   RefW:=50000; //band not exist for this control
  end;
end;

procedure TDRep.QRBand1AfterPrint(BandPrinted: Boolean);
begin

   SubFTableRealWidth:=FWordTable.FTableRealWidth;
   SubFFontNameTabs:=FWordParams.FFontNameTabs;
   SubFScaleFontSize:=FWordParams.FScaleFontSize;
   SubFFontSizeTabs:=FWordParams.FFontSizeTabs;
   SubTabA:=FTransferScale.TabA;
   SubTabB:=FTransferScale.TabB;
   SubMainA:=FTransferScale.MainA;
   SubMainB:=FTransferScale.MainB;
   SubShowExecute:=FShowExecute;
   SubIgnorePageHeader:=FWordParams.FIgnorePageHeader;
   SubIgnorePageFooter:=FWordParams.FIgnorePageFooter;

   SubTableAutoFormat:=FWordTable.TableAutoFormat;
   SubFTitleToTable:=FWordTable.FTitleToTable;
   SubFDetailToTable:=FWordTable.FDetailToTable;
   SubFPageHeaderToTable:=FWordTable.FPageHeaderToTable;
   SubFPageFooterToTable:=FWordTable.FPageFooterToTable;
   SubFGroupHeaderToTable:=FWordTable.FGroupHeaderToTable;
   SubFGroupFooterToTable:=FWordTable.FGroupFooterToTable;
   SubFSubDetailToTable:=FWordTable.FSubDetailToTable;
   SubFColumnHeaderToTable:=FWordTable.FColumnHeaderToTable;
   SubFOverlayToTable:=FWordTable.FOverlayToTable;
   SubFSummaryToTable:=FWordTable.FSummaryToTable;
   SubFSeparator:=FSeparator;
   SubFTableRealWidth:=FWordTable.FTableRealWidth;
   SubFDetailMultyRows:=FWordTable.FDetailMultyRows;
   SubFTableAutoFit:=FWordTable.FTableAutoFit;
   SubBorderInside:=FTabBorders.BorderInside;
   SubBorderOutside:=FTabBorders.BorderOutside;

   OnAftPrn; {calling transfer method}

end;

procedure TDRep.APrint(sender : TObject);
var   i : integer;
begin
 W.EndOfDocument;

 If FWordParams.FAutoClose then
 Begin
   W.FileSaveAs(Name := FWordParams.FFileName);
   W.AppClose;
 end
 else
 If Not FShowExecute then
 Begin
   W.AppShow;
   W.AppMaximize;
 end;

 I:=0;
 with fm do
 while (i<ComponentCount) do
 if (Components[i] is TQuickReport) then
 begin {calling old afterprint}
    (Components[i] As TQuickReport).AfterPrint:=AftPrn;
    If EXAP then
       (Components[i] As TQuickReport).AfterPrint(sender);
    Break;
 end
 else Inc(i);

end;


procedure TDRep.Loaded;
var frm : TComponent;
    i : Integer;
begin
 inherited Loaded;

 if Not(FExecute Or ExecuteToWord) then Exit;

 Ab1:=FStencil.FVerticalSensitiv;
 Ab2:=FStencil.FHorizontalSensitiv;
 TM:= FWordParams.FTopMargin;
 BM:= FWordParams.FBottomMargin;
 LM:= FWordParams.FLeftMargin;
 RM:= FWordParams.FRightMargin;

 PageA3:=0;
 EXBP:=False;
 EXAP:=False;
 BEvent:=TRUE;
 frm:=Owner;
 while (not (frm is TForm)) do frm:=frm.Owner;
 fm:=(frm as TForm);
 with fm do
 begin
  i:=0;
  while (i<ComponentCount) do
    if (Components[i] is TQuickReport) then
     begin
      tqr:=(Components[i] as TQuickReport);
      If Assigned((Components[i] as TQuickReport).BeforePrint) then
         EXBP:=True; {saving old call BeforePrint}
      If Assigned((Components[i] as TQuickReport).AfterPrint) then
         EXAP:=True; {saving old call AfterPrint}

      If Not (tqr.Orientation = poPortrait) then
         PageA3:=1;
      LM:=tqr.LeftMarginMM * 3+ FWordParams.FLeftMargin;
      FWordParams.FLeftMargin:=LM;
      AftPrn:=tqr.AfterPrint;
      BefPrn:=tqr.BeforePrint;
      tqr.BeforePrint:=BPrint;
      tqr.AfterPrint:=APrint;
      Break;
     end
    else Inc(i);
 end;
end;

procedure TDRep.BPrint(var PrintReport : Boolean);
Var I, J : Integer;
     str : string;
begin

  I:=0;
  with fm do
  while (i<ComponentCount) do
  if (Components[i] is TQuickReport) then
  begin {  beforeprint}
    (Components[i] As TQuickReport).BeforePrint:=BefPrn;
    If EXBP then
       (Components[i] As TQuickReport).BeforePrint(PrintReport);
    Break;
  end
  else Inc(i);


  if Not BEvent then exit;
  if  FExecute or ExecuteToWord then
  begin

   Ref:=0;
   L:=0;
  try
   W := CreateOleObject('Word.Basic');
  except
   raise EOleError.Create('Sorry!.. Can`t create the Word !!!');
  end;

   W.AppShow;
   W.AppMinimize;
//   ASav:=W.ToolsOptionsSave.AutoSave;

  If FShowExecute then
  Begin
//   W.AppShow;
   W.AppMaximize;
  end;

   PageWord:=SendPageWord.Create;
   PageWordOn:=SendPageWordOn.Create;

   W.FileNew;
//   W.ToolsOptionsSave(AutoSave:=1, FastSaves:=0);

   If PageA3 = 1 then W.TogglePortrait;
   W.FilePageSetup(TopMargin := TM,  BottomMargin := BM,
                  LeftMargin := LM, RightMargin := RM,
                  Orientation:= PageA3, ApplyPropsTo:=4,
                  HeaderDistance := FWordParams.FHeaderDistance,
                  FooterDistance := FWordParams.FFooterDistance);
   W.FormatParagraph(LineSpacingRule := 4,
                     LineSpacing := FWordParams.FLineSpacing);


   W.Font(FWordParams.FFontNameTabs, 8);

   J:=0;


   For I:=0 to fm.ComponentCount - 1 Do
   Begin
    If (fm.Components[I] Is TQRBand) then
    Begin   {   }
     PageWordOn.PrnBand[J]:=(fm.Components[I] As TQRBand).AfterPrint;
     (fm.Components[I] As TQRBand).AfterPrint:=QRBand1AfterPrint;
    end;

    If (fm.Components[I] Is TQRLabel) then
    Begin
     PageWord.OldTag[J]:=fm.Components[I].Tag;
     PageWord.Typ[J]:=1;
     Case ((fm.Components[I] As TQRLabel).Parent As TQRBand).BandType of
      rbTitle        : PageWord.Band[J]:=1;
      rbDetail       : PageWord.Band[J]:=2;
      rbPageHeader   : PageWord.Band[J]:=3;
      rbPageFooter   : PageWord.Band[J]:=4;
      rbGroupHeader  : PageWord.Band[J]:=5;
      rbGroupFooter  : PageWord.Band[J]:=6;
      rbSubDetail    : PageWord.Band[J]:=7;
      rbColumnHeader : PageWord.Band[J]:=8;
      rbOverlay      : PageWord.Band[J]:=9;
      rbSummary      : PageWord.Band[J]:=10;
     else
      PageWord.Band[J]:=10;
     end;
     fm.Components[I].Tag:=J; {Labels}
     PageWordOn.OldCall[J]:=TQRLabel(fm.Components[I]).OnPrint;
     TQRLabel(fm.Components[I]).OnPrint:=QRLabel1Print;
     Inc(J);
    end;
    If (fm.Components[I] Is TQRDBText) then
    Begin
     PageWord.OldTag[J]:=fm.Components[I].Tag;
     PageWord.Typ[J]:=2;
     Case ((fm.Components[I] As TQRDBText).Parent As TQRBand).BandType of
      rbTitle        : PageWord.Band[J]:=1;
      rbDetail       : PageWord.Band[J]:=2;
      rbPageHeader   : PageWord.Band[J]:=3;
      rbPageFooter   : PageWord.Band[J]:=4;
      rbGroupHeader  : PageWord.Band[J]:=5;
      rbGroupFooter  : PageWord.Band[J]:=6;
      rbSubDetail    : PageWord.Band[J]:=7;
      rbColumnHeader : PageWord.Band[J]:=8;
      rbOverlay      : PageWord.Band[J]:=9;
      rbSummary      : PageWord.Band[J]:=10;
     else
      PageWord.Band[J]:=10;
     end;
     fm.Components[I].Tag:=J; {DBTexts}
     PageWordOn.OldCall[J]:=TQRDBText(fm.Components[I]).OnPrint;
     TQRDBText(fm.Components[I]).OnPrint:=QRLabel1Print;
     Inc(J);
    end;
    If (fm.Components[I] Is TQRDBCalc) then
    Begin
     PageWord.OldTag[J]:=fm.Components[I].Tag;
     PageWord.Typ[J]:=3;
     Case ((fm.Components[I] As TQRDBCalc).Parent As TQRBand).BandType of
      rbTitle        : PageWord.Band[J]:=1;
      rbDetail       : PageWord.Band[J]:=2;
      rbPageHeader   : PageWord.Band[J]:=3;
      rbPageFooter   : PageWord.Band[J]:=4;
      rbGroupHeader  : PageWord.Band[J]:=5;
      rbGroupFooter  : PageWord.Band[J]:=6;
      rbSubDetail    : PageWord.Band[J]:=7;
      rbColumnHeader : PageWord.Band[J]:=8;
      rbOverlay      : PageWord.Band[J]:=9;
      rbSummary      : PageWord.Band[J]:=10;
     else
      PageWord.Band[J]:=10;
     end;
     fm.Components[I].Tag:=J; {DBCalc}
     PageWordOn.OldCall[J]:=TQRDBCalc(fm.Components[I]).OnPrint;
     TQRDBCalc(fm.Components[I]).OnPrint:=QRLabel1Print;
     Inc(J);
    end;
    If (fm.Components[I] Is TQRSysData) then
    Begin
     PageWord.OldTag[J]:=fm.Components[I].Tag;
     PageWord.Typ[J]:=4;
     Case ((fm.Components[I] As TQRSysData).Parent As TQRBand).BandType of
      rbTitle        : PageWord.Band[J]:=1;
      rbDetail       : PageWord.Band[J]:=2;
      rbPageHeader   : PageWord.Band[J]:=3;
      rbPageFooter   : PageWord.Band[J]:=4;
      rbGroupHeader  : PageWord.Band[J]:=5;
      rbGroupFooter  : PageWord.Band[J]:=6;
      rbSubDetail    : PageWord.Band[J]:=7;
      rbColumnHeader : PageWord.Band[J]:=8;
      rbOverlay      : PageWord.Band[J]:=9;
      rbSummary      : PageWord.Band[J]:=10;
     else
      PageWord.Band[J]:=10;
     end;
     fm.Components[I].Tag:=J; {SysData}
     PageWordOn.OldCall[J]:=TQRSysData(fm.Components[I]).OnPrint;
     TQRSysData(fm.Components[I]).OnPrint:=QRLabel1Print;
     Inc(J);
    end;
   end;
  // i:=0;
  end;
 BEvent:=FALSE;

end;

procedure Register;
begin
  RegisterComponents('QReport', [TDRep]);
end;

end.
